@@ -1,11 +1,10 @@
-from ccinput.packages.gaussian import GaussianCalculation
 from ccinput.tests.testing_utilities import InputTests
+from ccinput.exceptions import InvalidParameter
 
 class GaussianTests(InputTests):
 
     def test_sp_SE(self):
-        params = {
-                'nproc': 8,
+        params = { 'nproc': 8,
                 'mem': '10000MB',
                 'type': 'Single-Point Energy',
                 'in_file': 'Cl.xyz',
@@ -339,7 +338,7 @@ class GaussianTests(InputTests):
                 }
 
         with self.assertRaises(Exception):
-            inp = self.generate_calculation(**params)
+            self.generate_calculation(**params)
 
     def test_sp_DFT(self):
         params = {
@@ -587,7 +586,6 @@ class GaussianTests(InputTests):
                 'type': 'Geometrical Optimisation',
                 'in_file': 'Cl.xyz',
                 'software': 'Gaussian',
-                'method': 'AM1',
                 'charge': '-1',
                 'method': 'B3LYP',
                 'basis_set': '6-31+G(d,p)',
@@ -697,8 +695,6 @@ class GaussianTests(InputTests):
 
         self.assertTrue(self.is_equivalent(REF, inp.input_file))
 
-    #opt mod SE and HF
-
     def test_scan_bond_DFT(self):
         params = {
                 'nproc': 8,
@@ -993,7 +989,7 @@ class GaussianTests(InputTests):
                 }
 
         with self.assertRaises(Exception):
-            inp = self.generate_calculation(**params)
+            self.generate_calculation(**params)
 
     def test_no_method(self):
         params = {
@@ -1009,7 +1005,7 @@ class GaussianTests(InputTests):
                 }
 
         with self.assertRaises(Exception):
-            inp = self.generate_calculation(**params)
+            self.generate_calculation(**params)
 
     def test_freeze_dihedral_DFT(self):
         params = {
@@ -2166,7 +2162,7 @@ class GaussianTests(InputTests):
                 }
 
         with self.assertRaises(Exception):
-            inp = self.generate_calculation(**params)
+            self.generate_calculation(**params)
 
     def test_special_char(self):
         params = {
@@ -2333,3 +2329,339 @@ class GaussianTests(InputTests):
 
         self.assertTrue(self.is_equivalent(REF, inp.input_file))
 
+    def test_exchange_correlation_functional(self):
+        params = {
+                'nproc': 8,
+                'mem': '10GB',
+                'type': 'Single-Point Energy',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'pkzbpkzb',
+                'basis_set': '3-21G',
+                'charge': '-1',
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        %chk=calc.chk
+        %nproc=8
+        %mem=10000MB
+        #p sp PKZBPKZB/3-21G
+
+        File created by ccinput
+
+        -1 1
+        Cl 0.0 0.0 0.0
+
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_exchange_correlation_functional2(self):
+        params = {
+                'nproc': 8,
+                'mem': '10GB',
+                'type': 'Single-Point Energy',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'pBeHpkzb',
+                'basis_set': '3-21G',
+                'charge': '-1',
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        %chk=calc.chk
+        %nproc=8
+        %mem=10000MB
+        #p sp PBEhPKZB/3-21G
+
+        File created by ccinput
+
+        -1 1
+        Cl 0.0 0.0 0.0
+
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_restricted_HF(self):
+        params = {
+                'nproc': 8,
+                'mem': '10GB',
+                'type': 'Single-Point Energy',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'rHF',
+                'basis_set': '3-21G',
+                'charge': '-1',
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        %chk=calc.chk
+        %nproc=8
+        %mem=10000MB
+        #p sp RHF/3-21G
+
+        File created by ccinput
+
+        -1 1
+        Cl 0.0 0.0 0.0
+
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_unrestricted_HF(self):
+        params = {
+                'nproc': 8,
+                'mem': '10GB',
+                'type': 'Single-Point Energy',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'uHF',
+                'basis_set': '3-21G',
+                'charge': '-1',
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        %chk=calc.chk
+        %nproc=8
+        %mem=10000MB
+        #p sp UHF/3-21G
+
+        File created by ccinput
+
+        -1 1
+        Cl 0.0 0.0 0.0
+
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_unknown_HF(self):
+        params = {
+                'nproc': 8,
+                'mem': '10GB',
+                'type': 'Single-Point Energy',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'iHF',
+                'basis_set': '3-21G',
+                'charge': '-1',
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        %chk=calc.chk
+        %nproc=8
+        %mem=10000MB
+        #p sp iHF/3-21G
+
+        File created by ccinput
+
+        -1 1
+        Cl 0.0 0.0 0.0
+
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_unrestricted_DFT(self):
+        params = {
+                'nproc': 8,
+                'mem': '10GB',
+                'type': 'Single-Point Energy',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'uM062x',
+                'basis_set': '3-21G',
+                'charge': '-1',
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        %chk=calc.chk
+        %nproc=8
+        %mem=10000MB
+        #p sp UM062X/3-21G
+
+        File created by ccinput
+
+        -1 1
+        Cl 0.0 0.0 0.0
+
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_restricted_DFT(self):
+        params = {
+                'nproc': 8,
+                'mem': '10GB',
+                'type': 'Single-Point Energy',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'rPBE0',
+                'basis_set': '3-21G',
+                'charge': '-1',
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        %chk=calc.chk
+        %nproc=8
+        %mem=10000MB
+        #p sp RPBE1PBE/3-21G
+
+        File created by ccinput
+
+        -1 1
+        Cl 0.0 0.0 0.0
+
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_opt_freq(self):
+        params = {
+                'nproc': 8,
+                'mem': '10000MB',
+                'type': 'opt+freq',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'AM1',
+                'charge': '-1',
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        %chk=calc.chk
+        %nproc=8
+        %mem=10000MB
+        #p opt AM1 freq
+
+        File created by ccinput
+
+        -1 1
+        Cl 0.0 0.0 0.0
+
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_opt_freq2(self):
+        params = {
+                'nproc': 8,
+                'mem': '10000MB',
+                'type': 'optfreq',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'AM1',
+                'charge': '-1',
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        %chk=calc.chk
+        %nproc=8
+        %mem=10000MB
+        #p opt AM1 freq
+
+        File created by ccinput
+
+        -1 1
+        Cl 0.0 0.0 0.0
+
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_d3(self):
+        params = {
+                'nproc': 8,
+                'mem': '10GB',
+                'type': 'Single-Point Energy',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'M062X',
+                'basis_set': '3-21G',
+                'charge': '-1',
+                'd3': True,
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        %chk=calc.chk
+        %nproc=8
+        %mem=10000MB
+        #p sp M062X/3-21G EmpiricalDispersion=GD3
+
+        File created by ccinput
+
+        -1 1
+        Cl 0.0 0.0 0.0
+
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_d3bj(self):
+        params = {
+                'nproc': 8,
+                'mem': '10GB',
+                'type': 'Single-Point Energy',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'PBE0',
+                'basis_set': '3-21G',
+                'charge': '-1',
+                'd3bj': True,
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        %chk=calc.chk
+        %nproc=8
+        %mem=10000MB
+        #p sp PBE1PBE/3-21G EmpiricalDispersion=GD3BJ
+
+        File created by ccinput
+
+        -1 1
+        Cl 0.0 0.0 0.0
+
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_d3_d3bj_crash(self):
+        params = {
+                'nproc': 8,
+                'mem': '10GB',
+                'type': 'Single-Point Energy',
+                'in_file': 'Cl.xyz',
+                'software': 'Gaussian',
+                'method': 'PBE0',
+                'basis_set': '3-21G',
+                'charge': '-1',
+                'd3': True,
+                'd3bj': True,
+                }
+
+        with self.assertRaises(InvalidParameter):
+            self.generate_calculation(**params)
+
