@@ -1,6 +1,5 @@
 from ccinput.tests.testing_utilities import InputTests
-from ccinput.packages.orca import OrcaCalculation
-from ccinput.wrapper import generate_calculation
+from ccinput.exceptions import InvalidParameter
 
 class OrcaTests(InputTests):
 
@@ -341,7 +340,7 @@ class OrcaTests(InputTests):
                 }
 
         with self.assertRaises(Exception):
-            inp = self.generate_calculation(**params)
+            self.generate_calculation(**params)
 
     def test_sp_DFT(self):
         params = {
@@ -650,7 +649,7 @@ class OrcaTests(InputTests):
         H          0.94410        0.53240        2.04240
         *
         %geom Scan
-        B 0 1 = 9, 1.4, 10
+        B 0 1 = 9.00, 1.40, 10
         end
         end
         %pal
@@ -673,7 +672,7 @@ class OrcaTests(InputTests):
                 }
 
         with self.assertRaises(Exception):
-            inp = self.generate_calculation(**params)
+            self.generate_calculation(**params)
 
     def test_no_method(self):
         params = {
@@ -688,7 +687,7 @@ class OrcaTests(InputTests):
                 }
 
         with self.assertRaises(Exception):
-            inp = self.generate_calculation(**params)
+            self.generate_calculation(**params)
 
     def test_scan_angle_DFT(self):
         params = {
@@ -718,7 +717,7 @@ class OrcaTests(InputTests):
         H          0.94410        0.53240        2.04240
         *
         %geom Scan
-        A 1 0 2 = 9, 90, 10
+        A 1 0 2 = 9.00, 90.00, 10
         end
         end
         %pal
@@ -737,7 +736,7 @@ class OrcaTests(InputTests):
                 'charge': '0',
                 'method': 'B3LYP',
                 'basis_set': '6-31+G(d,p)',
-                'constraints': 'Scan_9_0_10/4_1_5_8;',
+                'constraints': 'Scan_9_1_10/4_1_5_8;',
                 }
 
         inp = self.generate_calculation(**params)
@@ -756,7 +755,7 @@ class OrcaTests(InputTests):
         H          0.94410        0.53240        2.04240
         *
         %geom Scan
-        D 3 0 4 7 = 9, 0, 10
+        D 3 0 4 7 = 9.00, 1.00, 10
         end
         end
         %pal
@@ -1184,7 +1183,7 @@ class OrcaTests(InputTests):
         H         -1.82448        0.94856        3.28105
         *
         %neb
-        product "struct2.xyz"
+        product "calc2.xyz"
         nimages 8
         end
         %pal
@@ -1222,7 +1221,46 @@ class OrcaTests(InputTests):
         H         -1.82448        0.94856        3.28105
         *
         %neb
-        product "struct2.xyz"
+        product "calc2.xyz"
+        nimages 12
+        end
+        %pal
+        nprocs 8
+        end
+
+        """
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_NEB_aux_name(self):
+        params = {
+                'nproc': 8,
+                'type': 'Minimum Energy Path',
+                'in_file': 'elimination_substrate.xyz',
+                'auxiliary_file': 'elimination_product.xyz',
+                'software': 'ORCA',
+                'specifications': '--nimages 12',
+                'charge': -1,
+                'method': 'gfn2-xtb',
+                'aux_name': 'product',
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """!NEB xtb2
+        *xyz -1 1
+        C         -0.74277        0.14309        0.12635
+        C          0.71308       -0.12855       -0.16358
+        Cl         0.90703       -0.47793       -1.61303
+        H         -0.84928        0.38704        1.20767
+        H         -1.36298       -0.72675       -0.06978
+        H         -1.11617        0.99405       -0.43583
+        H          1.06397       -0.95639        0.44985
+        H          1.30839        0.75217        0.07028
+        O         -0.91651        0.74066        3.00993
+        H         -1.82448        0.94856        3.28105
+        *
+        %neb
+        product "product.xyz"
         nimages 12
         end
         %pal
@@ -1245,7 +1283,6 @@ class OrcaTests(InputTests):
                 }
 
         inp = self.generate_calculation(**params)
-
         REF = """
         !SP M062X Def2-SVP
         *xyz -1 1
@@ -1350,3 +1387,94 @@ class OrcaTests(InputTests):
 
         self.assertTrue(self.is_equivalent(REF, inp.input_file))
 
+    def test_opt_freq(self):
+        params = {
+                'nproc': 8,
+                'type': 'opt+freq',
+                'in_file': 'Cl.xyz',
+                'software': 'ORCA',
+                'method': 'AM1',
+                'charge': '-1',
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        !OPT FREQ AM1
+        *xyz -1 1
+        Cl 0.0 0.0 0.0
+        *
+        %pal
+        nprocs 1
+        end
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_d3(self):
+        params = {
+                'nproc': 8,
+                'type': 'opt',
+                'in_file': 'Cl.xyz',
+                'software': 'ORCA',
+                'basis_set': 'Def2TZVP',
+                'method': 'M062X',
+                'charge': '-1',
+                'd3': True,
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        !OPT M062X Def2-TZVP d3zero
+        *xyz -1 1
+        Cl 0.0 0.0 0.0
+        *
+        %pal
+        nprocs 8
+        end
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_d3bj(self):
+        params = {
+                'nproc': 8,
+                'type': 'opt',
+                'in_file': 'Cl.xyz',
+                'software': 'ORCA',
+                'basis_set': 'Def2TZVP',
+                'method': 'PBE0',
+                'charge': '-1',
+                'd3bj': True,
+                }
+
+        inp = self.generate_calculation(**params)
+
+        REF = """
+        !OPT PBE0 Def2-TZVP d3bj
+        *xyz -1 1
+        Cl 0.0 0.0 0.0
+        *
+        %pal
+        nprocs 8
+        end
+        """
+
+        self.assertTrue(self.is_equivalent(REF, inp.input_file))
+
+    def test_d3_d3bj_crash(self):
+        params = {
+                'nproc': 8,
+                'type': 'opt',
+                'in_file': 'Cl.xyz',
+                'software': 'ORCA',
+                'basis_set': 'Def2TZVP',
+                'method': 'PBE0',
+                'charge': '-1',
+                'd3': True,
+                'd3bj': True,
+                }
+
+        with self.assertRaises(InvalidParameter):
+            self.generate_calculation(**params)
