@@ -1,10 +1,13 @@
 import os
 
+from ccinput.__init__ import __version__
 from ccinput.packages.gaussian import GaussianCalculation
 from ccinput.packages.orca import OrcaCalculation
 
-from ccinput.calculation import Calculation, Parameters
-from ccinput.utilities import get_abs_type, get_abs_software, standardize_xyz, parse_xyz_from_file
+from ccinput.calculation import Calculation, Parameters, Constraint, parse_str_constraints, \
+                                parse_freeze_constraints, parse_scan_constraints
+from ccinput.utilities import get_abs_type, get_abs_software, standardize_xyz, \
+                              parse_xyz_from_file
 from ccinput.exceptions import *
 
 SOFTWARE_CLASSES = {
@@ -18,9 +21,10 @@ def process_calculation(calc):
 
 def generate_calculation(software=None, type=None, method="", basis_set="", \
             solvent="", solvation_model="", solvation_radii="",  specifications="", \
+            freeze=[], scan=[], sfrom=[], sto=[], snsteps=[], sstep=[], \
             density_fitting="", custom_basis_sets="", xyz="", in_file="", \
-            constraints="", nproc=0, mem="", charge=0, multiplicity=1, name="calc", \
-            header="File created by ccinput", **kwargs):
+            constraints="", nproc=0, mem="", charge=0, multiplicity=1, d3=False, d3bj=False, \
+            aux_name="calc2", name="calc", header="File created by ccinput", **kwargs):
 
     if software is None:
         raise InvalidParameter("Specify a software package to use (software=...)")
@@ -33,7 +37,7 @@ def generate_calculation(software=None, type=None, method="", basis_set="", \
     elif in_file != "":
         xyz_structure = parse_xyz_from_file(in_file)
     else:
-        raise InvalidParameter("No input")
+        raise InvalidParameter("No input structure")
 
     abs_software = get_abs_software(software)
 
@@ -41,11 +45,16 @@ def generate_calculation(software=None, type=None, method="", basis_set="", \
 
     params = Parameters(abs_software, solvent, solvation_model, solvation_radii, \
             basis_set, method, specifications, density_fitting, \
-            custom_basis_sets, **kwargs)
+            custom_basis_sets, d3, d3bj, **kwargs)
 
-    calc = Calculation(xyz_structure, params, calc_type, constraints=constraints, \
+    _constraints = parse_str_constraints(constraints, xyz_structure, software=abs_software)
+    _constraints += parse_freeze_constraints(freeze, xyz_structure, software=abs_software)
+    _constraints += parse_scan_constraints(scan, sfrom, sto, snsteps, sstep,
+                                           xyz_structure, software=abs_software)
+
+    calc = Calculation(xyz_structure, params, calc_type, constraints=_constraints, \
             nproc=nproc, mem=mem, charge=charge, multiplicity=multiplicity, \
-            name=name, header=header, software=abs_software)
+            aux_name=aux_name, name=name, header=header, software=abs_software)
 
     return process_calculation(calc)
 
@@ -57,9 +66,9 @@ def write_input(filename, **args):
     with open(filename, 'w') as out:
         out.write(inp)
 
-def cmd():
+def get_parser():
     import argparse
-    parser = argparse.ArgumentParser(description='Generation an input for a computational chemistry package')
+    parser = argparse.ArgumentParser(description='Generates an input for a computational chemistry package')
     parser.add_argument('software', help='Desired software package (Gaussian or ORCA)')
 
     parser.add_argument('type', help='Calculation type (opt, freq, sp, ...)')
@@ -98,6 +107,24 @@ def cmd():
     parser.add_argument('--constraints', '-co', default="",
             type=str, help='String specification of constraints for certain calculations')
 
+    parser.add_argument('--freeze', action="append", nargs="+", default=[], metavar="ATOM",
+            help='Freeze specified distance/angle/dihedral between the specified atoms')
+
+    parser.add_argument('--scan', action="append", nargs="+", default=[], metavar="ATOM",
+            help='Scan specified distance/angle/dihedral between the specified atoms (additional parameters required)')
+
+    parser.add_argument('--from', action="append", dest='sfrom', default=[], metavar="FROM",
+            help='Initial distance (for --scan)')
+
+    parser.add_argument('--to', action="append", dest='sto', default=[], metavar="TO",
+            help='Final distance (for --scan)')
+
+    parser.add_argument('--nsteps', action="append", dest='snsteps', default=[], metavar="NSTEPS",
+            help='Number of steps (for --scan)')
+
+    parser.add_argument('--step', action="append", dest='sstep', default=[], metavar="STEP",
+            help='Step size in Å (for --scan)')
+
     parser.add_argument('--nproc', '-n', default=1,
             type=int, help='Number of CPU cores to use')
 
@@ -108,24 +135,52 @@ def cmd():
 
     parser.add_argument('--mult', '-m', default=1, type=int, help='Multiplicity of the system')
 
+    dispersion_group = parser.add_mutually_exclusive_group()
+
+    dispersion_group.add_argument('--d3', action='store_true', help="Use the D3(0) dispersion "
+            "correction")
+
+    dispersion_group.add_argument('--d3bj', action='store_true', help="Use the D3 dispersion "
+            "correction with Becke-Johnson damping")
+
     parser.add_argument('--name', default="calc", type=str,
             help='Name of the produced file (unused by some packages)')
 
+    parser.add_argument('--aux_name', default="calc2", type=str,
+            help='Name of the auxiliary file (some calculation types only)')
+
     parser.add_argument('--header', default="File created by ccinput", type=str,
             help='Header in produced file (unused by some packages)')
 
+    parser.add_argument('--version', '-v', action='version', version=f'%(prog)s {__version__}')
+    return parser
+
+def cmd():
+    parser = get_parser()
     args = parser.parse_args()
 
-    inp = gen_input(software=args.software, type=args.type, method=args.method, \
-            basis_set=args.basis_set, solvent=args.solvent, solvation_model=args.solvation_model, \
-            solvation_radii=args.solvation_radii,  specifications=args.specifications, density_fitting=args.density_fitting, \
-            custom_basis_sets=args.custom_basis_sets, xyz=args.xyz, in_file=args.file, constraints=args.constraints, \
-            nproc=args.nproc, mem=args.mem, charge=args.charge, multiplicity=args.mult, \
-            name=args.name, header=args.header)
-
+    inp = get_input_from_args(args)
     if args.output != "":
         with open(args.output, 'w') as out:
             out.write(inp)
-        print("Input file written to {}".format(args.output))
+        print(f"Input file written to {args.output}")
     else:
         print(inp)
+
+def get_input_from_args(args):
+    try:
+        inp = gen_input(software=args.software, type=args.type, method=args.method, \
+                basis_set=args.basis_set, solvent=args.solvent, \
+                solvation_model=args.solvation_model, solvation_radii=args.solvation_radii, \
+                specifications=args.specifications, freeze=args.freeze, \
+                scan=args.scan, sfrom=args.sfrom, sto=args.sto, snsteps=args.snsteps, \
+                sstep=args.sstep, density_fitting=args.density_fitting, \
+                custom_basis_sets=args.custom_basis_sets, xyz=args.xyz, in_file=args.file, \
+                constraints=args.constraints, nproc=args.nproc, mem=args.mem, \
+                charge=args.charge, multiplicity=args.mult, d3=args.d3, d3bj=args.d3bj, \
+                aux_name=args.aux_name, name=args.name, header=args.header)
+    except CCInputException as e:
+        print(f"*** {str(e)} ***")
+        return
+    return inp
+
